@@ -1,5 +1,6 @@
 
 from lofarSun.IM import IMdata
 from lofarSun.BF import BFdata
+from lofarSun import cli
 
-__all__ = ['IMdata','BFdata','util']
\ No newline at end of file
+__all__ = ['IMdata','BFdata','cli']
\ No newline at end of file
